function varargout = METAlab_GTG(varargin)

% Author: Jeffrey M. Spielberg (jspielb2@gmail.com)
% Version: Beta 0.34 (10.06.14)
% 
% 
% History:
% 02.27.14 - Beta 0.13 - initial public release
% 03.11.14 - Beta 0.20 - 1) small bugfixes, 2) major overhaul of user 
%                        interface into GUIs, 3) addition of testing of 
%                        negative weights in thresholded matrices, 4) 
%                        addition of testing of only non-disconnected 
%                        matrices for thresholded matrices, 5) addition of 
%                        pagerank & eigenvector centrality
% 03.17.14 - Beta 0.21 - 1) small bugfixes, 2) addition of parallel
%                        computing toolbox to stage 3
% 03.24.14 - Beta 0.22 - lots of small bugfixes
% 04.08.14 - Beta 0.23 - 1) lots of small bugfixes, 2) addition of F-tests
%                        across multiple predictors (particularly useful
%                        for categorical predictors) in the 4th stage, 
%                        3) addition of calpability for more than 2 within-
%                        participant levels (although, only one within 
%                        factor is allowed for now)
% 04.23.14 - Beta 0.24 - 1) lots of small bugfixes, 2) conversion of Stage 
%                        2 into GUI format, 3) addition of calpability in 
%                        Stage 2 for creating connectivity matrices by 
%                        condition for task fMRI
% 05.07.14 - Beta 0.25 - 1) addition of output file for significant
%                        effects, 2) addition of k-coreness centrality 
%                        testing, 3) addition of option to do detrending by
%                        block for functional data in Stage 2
% 06.10.14 - Beta 0.30 - 1) small bugfixes, 2) all properties previously 
%                        calculated/tested for all weights simultaneously 
%                        are now calculated/tested for each sign separately 
%                        (modularity still calculated on entire matrix), 3)
%                        clustering coefficient, local efficiency, matching
%                        index, rich club, & transitivity added to full 
%                        network calculations/testing, 4) participant 
%                        coefficient & within-module z added to thresholded
%                        calculations/testing, 5) calculation/testing of 
%                        properties for negative weights in the thresholded
%                        matrices is now automatic (to be consistent with 
%                        fully connected matrices), 6) option added to 
%                        calculate/test properties for absolute value of 
%                        weights, 7) handles now used in all stages to 
%                        pass information between functions (rather than 
%                        via out, which was made global), allowing users to
%                        launch processes from the same gui with less 
%                        chance of info from the previous process 
%                        interfering, 8) important change to the way in
%                        which matrices are thresholded in Stage 3; the
%                        previous version allowed the threshold to be
%                        negative, which sometimes happened for higher
%                        densities (this was a problem for negative weight
%                        properties, but is unlikely to have affected
%                        properties for positive weights); thus, when
%                        calculating properties for negative weights,
%                        sometime positive weights were also allowed in at
%                        higher densities, which it more likely for 
%                        properties for positive and negative weights to be
%                        negatively correlated; now, only weights of the
%                        appropriate sign are allowed in; as a consequence,
%                        it is possible that the chosen maximum density 
%                        cannot be reached (less likely for positive
%                        weights, but is generally problematic for
%                        'sparser' matrices; therefore, the maximum 
%                        possible density is now calculated and, if smaller
%                        than the requested density, used
% 06.25.14 - Beta 0.31 - 1) addition to Stage 1 of Muschelli et al. 
%                        (2014)'s procedure for partialing the first 5 
%                        principal components instead of the mean of white 
%                        matter and ventricular signal (NOTE, local white 
%                        matter partialling is not currently computed if 
%                        the PCA option is selected), 2) addition to Stage 
%                        1 of Patel et al. (2014)'s WaveletDespike 
%                        procedure, 3) addition to stages 3 & 4 of 
%                        calculation & testing of mean (across the network)
%                        clustering coefficient and local efficiency
% 08.26.14 - Beta 0.32 - 1) small bugfixes, 2) fixed bug in Stage 1 whereby
%                        wavelet despike was saving files in RAS format,
%                        but the script expected LAS; now the LR
%                        orientation is checked & changed to LAS if need be
%                        3) addition of calculating & testing properties 
%                        from binarized matrices, 4) Stage 1 now compatible 
%                        with both compressed & non-compressed nifti 
%                        format, 5)addition of least trimmed squares 
%                        regression (LTS) to Stage 4, 6) code 
%                        simplification in Stage 4
% 09.22.14 - Beta 0.33 - 1) small bugfixes in Stage 4
% 10.06.14 - Beta 0.34 - Stage 1 changes: 1) ROIlabel cell array must now 
%                        contain 2 columns: the 1st contains the ROI labels
%                        and the 2nd the numeric identifiers corresponding 
%                        to the 3d ROI image. This allows for the case that
%                        some participants may not have all ROIs, in which
%                        case NaNs will be used for the timeseries for this
%                        ROI, 2) added two new possible methods for
%                        extracting timeseries from each ROI (previously
%                        only the mean was available): median, largest
%                        principal component, 3) added the capability to
%                        erode white matter and ventricle masks
%
% 
% WARNING: This is a beta version. There no known bugs, but only limited 
% testing has been performed. This software comes with no warranty (even 
% the implied warranty of merchantability or fitness for a particular 
% purpose). Therefore, USE AT YOUR OWN RISK!!!
%
% Copyleft 2014. Software can be modified and redistributed, but modified, 
% redistributed versions must have the same rights


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Still To Do %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 1. Add calpability for more than one repeated factor (low priority)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 2. Add functionality for directed matrices (low priority)
% 2a. Add tests of direction/causality
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% Begin initialization code
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
    'gui_Singleton',  gui_Singleton, ...
    'gui_OpeningFcn', @METAlab_GTG_OpeningFcn, ...
    'gui_OutputFcn',  @METAlab_GTG_OutputFcn, ...
    'gui_LayoutFcn',  [] , ...
    'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code

function METAlab_GTG_OpeningFcn(hObject, ~, handles, varargin)
handles.output = hObject;
guidata(hObject, handles);

function varargout = METAlab_GTG_OutputFcn(~, ~, handles)
varargout{1} = handles.output;




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% Preprocess timeseries data %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function preproc_timeseries_Callback(~,~,~)
METAlab_GTG_preprocess_GUI;




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% Create connectivity matrices %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function create_conmats_Callback(~,~,~)
METAlab_GTG_createconmats_GUI;




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% Calculate graph theoretic properties %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function calc_graph_props_Callback(~,~,~)
METAlab_GTG_calcproperties_GUI;




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% Run permutation analyses on network properties %%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function run_perm_analyses_Callback(~,~,~)
METAlab_GTG_runpermanalyses_GUI;




% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %%%% Add logo to GUI %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function Logo_axes_CreateFcn(hObject, eventdata, handles) %#ok<*DEFNU,*INUSD>
axes(hObject);

r = uint8([255,255,255,255,255,255,255,255,255,255,255,255,254,252,255,255,255,255,255,255,254,255,255,255,255,255,255,253,252,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,255,255,255,255,255,255,255,255,252,253,255,255,255,224,182,178,176,213,255,243,187,177,177,199,241,255,255,255,253,253,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,255,255,255,255,255,255,253,254,254,255,238,129,54,16,0,1,0,3,199,107,0,3,0,4,25,72,180,255,255,255,253,254,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,255,255,255,255,255,254,255,248,153,70,29,0,0,0,1,2,3,0,90,46,0,4,2,1,0,0,8,39,84,192,255,252,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,255,255,255,255,253,255,210,64,0,1,0,3,2,1,0,0,1,0,65,32,0,1,0,1,1,3,2,0,0,6,112,245,255,254,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,255,255,255,253,255,233,30,0,5,3,1,0,0,0,0,0,1,0,65,32,0,1,0,0,0,0,0,2,3,5,0,102,255,251,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,255,255,255,254,255,101,0,5,0,0,0,0,0,0,0,0,1,0,45,27,0,1,0,0,0,0,0,0,0,1,2,3,174,255,253,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,255,255,252,255,149,0,4,0,0,0,0,0,0,0,0,0,1,0,39,24,0,1,0,0,0,0,0,0,0,0,2,0,20,217,255,253,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,255,255,253,254,50,0,3,0,0,0,0,0,0,0,0,0,1,0,28,20,0,1,0,0,0,0,0,0,0,0,0,5,0,139,255,251,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,255,253,255,223,22,0,1,0,0,0,0,0,0,0,0,0,1,0,29,20,0,1,0,0,0,0,0,0,0,0,0,3,0,88,255,254,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,255,255,253,71,0,3,0,0,0,0,0,0,0,0,0,0,0,0,19,12,0,0,0,0,0,0,0,0,0,0,0,0,2,0,141,255,252,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,252,255,193,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,14,8,0,0,0,0,0,0,0,0,0,0,0,0,2,0,41,247,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,252,255,149,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,2,1,0,0,0,0,0,0,0,0,0,0,0,0,1,0,12,213,255,253,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,253,255,94,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,159,255,252,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,253,255,209,11,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,59,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,252,255,176,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,23,238,255,254,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,253,255,101,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,170,255,252,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,254,253,253,253,252,254,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,254,253,253,253,252,253,255,255,255,255,255,255,255,255,255,255,255;254,255,247,34,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,109,255,252,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,254,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;254,255,236,24,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,105,255,250,251,251,251,251,251,251,254,255,255,255,255,255,255,255,255,255,253,251,251,251,251,251,252,255,255,252,251,251,251,251,251,251,251,251,251,251,251,251,251,251,255,255,252,251,251,251,251,251,251,251,251,251,251,251,251,251,251,251,252,255,255,255,255,255,255,255,255,254,254,254,255,255,255,255,255,255,255,232,219,217,208,191,224,255,254,255,255,255,255,255,255,255,255,255,255,255,255,255,241,218,219,211,196,208,255,255,255,255,255,255,255,255,255,255,255;254,255,244,33,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,99,255,252,255,255,255,255,255,255,254,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,254,251,254,255,255,255,255,255,253,255,131,20,4,6,0,130,255,252,255,255,255,255,255,255,255,255,255,255,255,253,255,181,40,11,5,0,66,254,254,255,255,255,255,255,255,255,255,255;253,255,201,7,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,48,255,178,83,107,110,110,110,108,224,255,254,255,255,255,255,255,253,255,207,101,111,109,108,93,112,248,255,146,87,108,110,109,107,95,95,95,96,96,100,111,107,121,242,255,144,101,113,105,96,96,96,107,110,107,96,96,96,104,112,102,141,255,254,255,255,255,255,252,255,170,60,237,255,254,255,255,255,255,255,241,214,88,0,0,132,255,252,255,255,255,255,255,255,255,255,255,255,255,255,255,247,229,151,5,0,70,254,254,255,255,255,255,255,255,255,255,255;252,255,181,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,24,244,196,86,22,0,1,4,0,139,255,252,255,255,255,255,255,252,255,124,0,4,0,4,72,137,250,255,168,75,13,0,0,18,108,111,107,93,92,45,0,0,23,231,255,58,0,0,18,80,94,100,13,0,10,98,95,86,25,0,0,51,255,254,255,255,255,255,253,255,75,0,169,255,252,255,255,255,255,255,253,255,213,17,0,132,255,252,255,255,255,255,255,255,255,255,255,255,255,255,255,254,255,254,64,0,70,255,254,255,255,255,255,255,255,255,255,255;252,255,99,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,160,255,255,230,33,15,22,0,53,253,254,255,255,255,255,255,254,254,52,0,10,0,129,255,252,255,255,253,255,198,13,0,50,253,255,254,255,254,255,122,0,32,233,255,68,0,29,216,255,255,246,41,0,35,241,255,255,234,52,0,62,255,254,255,255,255,253,255,207,14,0,76,255,253,255,255,255,255,255,253,255,222,24,0,133,255,252,255,255,255,255,255,255,255,255,255,255,255,255,255,255,253,254,74,0,70,255,254,255,255,255,255,255,255,255,255,255;255,255,53,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,116,255,245,255,106,51,104,0,4,195,255,252,255,255,255,252,255,190,35,87,0,0,170,255,249,255,255,252,255,233,29,0,45,245,252,251,252,252,253,255,64,14,235,255,68,0,176,255,249,255,241,36,0,31,236,255,249,255,197,0,59,255,254,255,255,255,252,255,124,0,3,12,213,255,253,255,255,255,255,254,255,221,23,0,133,255,252,255,255,255,255,255,254,254,254,254,255,255,255,255,255,254,255,72,0,70,254,254,254,254,254,255,255,255,255,255,255;255,247,32,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,98,255,249,255,124,50,191,0,0,112,255,252,255,255,255,252,255,91,85,136,0,0,170,255,251,255,255,254,255,229,26,0,45,248,255,254,255,255,252,255,185,35,235,255,45,58,255,252,254,255,244,37,0,31,239,255,254,251,255,87,36,255,254,255,255,254,255,246,43,36,3,0,130,255,252,255,255,255,255,254,255,222,23,0,133,255,252,255,255,255,253,253,255,255,255,255,252,255,255,255,255,254,255,73,0,70,254,251,254,255,255,253,253,255,255,255,255;255,243,26,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,89,255,249,255,125,36,243,43,0,34,239,255,254,255,254,255,236,17,184,129,0,0,170,255,251,255,255,254,255,231,27,0,45,248,255,254,255,254,237,251,254,234,251,255,136,187,255,253,254,255,244,37,0,31,239,255,254,253,255,203,132,255,254,255,255,252,255,172,33,192,19,0,46,248,255,254,255,255,255,254,255,222,23,0,133,255,252,255,255,253,255,255,243,229,238,253,255,254,255,255,255,254,255,73,0,69,255,255,252,237,245,255,255,253,255,255,255;255,212,10,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,39,245,254,255,127,31,255,120,0,0,168,255,252,255,252,255,152,29,255,119,0,0,170,255,251,255,255,254,255,231,27,0,45,248,254,253,255,238,45,214,255,254,255,255,255,254,254,255,254,255,244,37,0,31,239,255,254,255,255,255,255,255,255,255,255,253,255,76,90,255,71,0,0,184,255,252,255,255,255,254,255,222,23,0,133,255,252,255,254,255,199,86,34,30,15,41,136,249,255,254,255,254,255,73,0,77,244,125,35,2,20,87,202,255,253,255,255;255,181,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,17,235,255,255,126,33,255,204,5,0,82,255,253,255,253,255,57,115,255,113,0,0,170,255,251,255,255,254,255,231,27,0,46,249,255,252,255,177,1,216,255,253,255,255,252,253,255,255,254,255,244,37,0,31,239,255,254,255,255,253,252,255,255,255,254,255,223,13,188,255,164,0,0,96,255,253,255,255,255,254,255,222,23,0,133,255,252,253,255,157,0,66,217,233,166,13,0,110,255,253,255,254,255,72,0,48,79,77,136,120,35,0,11,170,255,253,255;255,176,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,8,222,255,255,126,35,255,254,62,0,17,219,255,251,255,203,18,210,255,115,0,0,170,255,251,255,255,254,255,231,27,0,44,244,247,250,220,47,0,218,255,253,255,255,255,255,255,255,254,255,244,37,0,31,239,255,254,255,255,255,255,255,255,255,251,255,133,42,252,255,235,35,0,24,225,255,254,255,255,254,255,222,23,0,133,255,250,255,228,21,0,114,255,249,255,137,0,17,216,255,253,254,255,71,0,15,180,255,252,255,237,62,0,13,209,255,253;255,175,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,4,217,255,255,126,35,253,255,146,0,0,141,255,248,255,106,63,255,255,116,0,0,170,255,251,255,255,254,255,231,25,0,7,32,30,25,7,0,7,216,255,253,255,255,255,255,255,255,254,255,244,37,0,31,239,255,254,255,255,255,255,255,255,255,255,255,44,129,255,245,255,111,0,0,150,255,252,255,255,254,255,222,23,0,133,255,250,255,220,12,0,95,255,247,255,190,0,0,179,255,251,254,255,73,0,73,255,252,250,249,255,204,11,0,92,255,253;255,186,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,24,240,255,255,126,35,255,255,224,18,0,56,252,255,245,28,165,255,255,116,0,0,170,255,251,255,255,254,255,231,26,0,34,191,191,180,134,14,5,217,255,253,255,255,255,255,255,255,254,255,244,37,0,31,239,255,254,255,255,255,255,255,255,253,255,187,24,234,255,251,255,213,11,0,61,255,254,255,255,254,255,222,23,0,133,255,252,254,255,167,117,224,255,250,255,194,1,0,172,255,251,254,255,73,0,69,252,254,254,255,251,255,68,0,22,233,255;255,191,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,19,234,255,255,126,35,255,252,255,84,0,6,190,255,161,33,243,254,255,116,0,0,170,255,251,255,255,254,255,231,27,0,48,252,255,251,255,137,0,219,255,254,254,254,255,255,255,255,254,255,244,37,0,31,239,255,254,255,255,255,255,255,255,252,255,96,33,177,165,167,166,167,34,0,6,197,255,253,255,254,255,222,23,0,133,255,252,255,255,255,255,254,255,232,164,56,0,0,173,255,251,254,255,73,0,70,255,254,255,255,251,255,118,0,4,192,255;255,177,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,12,228,255,255,126,36,255,251,255,172,0,0,109,255,65,113,255,248,255,116,0,0,170,255,251,255,255,254,255,231,27,0,45,245,252,250,255,220,12,213,255,253,255,255,255,255,255,255,254,255,244,37,0,31,239,255,254,255,255,255,255,255,254,255,234,26,27,28,26,26,25,29,30,3,0,113,255,252,255,254,255,222,23,0,133,255,252,255,253,255,244,185,86,43,82,124,11,0,173,255,251,254,255,73,0,70,255,254,255,255,252,255,139,0,0,176,255;255,178,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,13,230,255,255,127,34,255,254,255,238,36,0,41,207,22,210,255,250,255,116,0,0,170,255,251,255,255,254,255,231,27,0,45,248,255,254,255,249,162,236,255,255,223,230,255,254,255,255,254,255,244,37,0,31,239,255,254,255,255,255,255,255,252,255,150,28,241,255,252,253,252,255,226,27,0,32,240,255,254,254,255,222,23,0,133,255,252,253,255,188,49,0,88,215,255,200,6,0,173,255,251,254,255,73,0,70,255,254,255,255,252,255,139,0,0,178,255;255,219,13,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,52,250,253,255,128,26,247,255,250,255,111,0,16,44,77,255,254,252,255,116,0,0,170,255,251,255,255,254,255,231,27,0,46,249,255,254,255,255,255,255,254,255,47,139,255,252,255,255,254,255,244,37,0,31,239,255,254,255,255,255,255,255,255,255,50,106,255,251,255,255,255,252,255,96,0,0,168,255,252,254,255,222,23,0,133,255,250,255,199,13,0,126,255,250,255,189,0,0,173,255,251,254,255,73,0,69,254,254,255,255,251,255,124,0,10,205,255;255,255,46,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,100,255,249,255,129,21,241,255,252,255,195,0,3,0,166,255,252,252,255,116,0,0,169,255,252,255,255,254,255,230,26,0,44,246,255,254,255,255,252,251,255,184,0,166,255,252,255,255,254,255,243,36,0,31,238,255,254,255,255,255,255,253,255,199,0,191,255,253,255,255,255,252,255,182,0,0,79,255,253,254,255,221,23,0,132,255,248,255,96,0,25,231,255,249,255,193,1,0,172,255,251,254,255,73,0,72,254,254,255,255,251,255,83,0,44,250,255;253,255,82,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,134,255,245,255,110,0,219,255,249,255,249,57,0,37,241,255,252,248,255,120,0,0,170,255,247,255,255,251,255,233,28,0,38,239,255,251,251,252,252,255,252,62,0,183,255,252,255,255,251,254,244,39,0,27,238,255,250,254,255,255,253,250,255,97,21,250,253,252,255,255,255,254,253,241,43,0,11,212,255,248,255,221,24,0,134,255,249,255,71,0,37,247,254,248,255,191,0,0,176,255,250,254,255,72,0,55,248,253,252,251,255,227,25,0,146,255,252;252,255,163,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,8,214,255,255,234,41,0,114,255,254,251,255,140,0,123,255,251,255,254,255,84,0,0,120,255,252,255,255,253,255,214,17,1,11,199,255,253,255,255,253,255,130,0,0,194,255,252,255,255,253,255,225,21,1,5,191,255,253,255,255,255,253,255,186,2,42,244,255,254,255,255,255,255,251,255,104,0,0,92,255,253,255,218,22,0,106,255,249,255,86,0,10,198,255,255,226,64,0,0,154,255,252,254,255,72,0,0,169,255,252,253,255,111,0,73,250,255,254;254,255,246,57,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,109,255,176,80,25,0,2,0,31,103,229,255,215,14,205,255,255,186,101,61,0,1,0,2,63,129,249,255,161,94,35,0,2,0,16,80,88,100,95,80,35,0,1,0,206,255,252,255,241,124,93,27,0,2,0,9,67,119,238,255,255,171,65,5,0,0,33,89,183,255,252,255,242,128,76,11,0,1,0,40,125,143,58,0,0,7,84,204,255,186,6,0,21,99,90,52,178,54,0,19,87,180,255,253,62,6,34,2,123,200,193,88,0,75,239,255,254,255;255,253,255,216,16,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,48,244,255,176,107,130,133,132,134,126,118,228,255,250,211,251,254,255,184,103,125,133,132,132,133,117,128,249,255,158,107,128,133,132,133,129,119,118,115,116,120,126,133,132,134,238,255,253,255,241,123,114,128,133,132,133,131,119,121,237,255,255,168,110,134,133,132,128,106,181,255,252,255,242,126,116,130,132,132,134,122,127,121,118,133,132,134,106,195,255,255,182,66,37,37,98,220,255,204,88,66,74,195,255,252,48,123,250,143,64,54,49,60,160,251,254,254,255,255;255,255,252,255,98,0,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,144,255,251,255,255,255,255,255,255,255,255,255,255,255,255,254,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,253,255,251,244,252,255,255,253,255,255,255,255,255,255,255,226,245,255,255,255,244,244,255,255,253,254,255,255,255;255,255,254,255,233,55,0,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,0,108,255,254,255,253,251,251,251,251,251,251,251,254,255,255,253,255,255,255,253,251,251,251,251,251,251,251,251,255,255,252,251,251,251,251,251,251,251,251,251,251,251,251,251,251,251,254,255,255,255,255,251,251,251,251,251,251,251,251,251,254,255,255,253,251,251,251,251,251,251,253,255,255,255,255,251,251,251,251,251,251,251,251,251,251,251,251,251,251,253,255,255,252,253,255,254,251,254,255,253,252,254,251,253,255,255,255,254,254,251,253,255,255,253,252,255,255,255,255,255;255,255,255,254,255,242,59,0,5,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,5,0,89,255,254,254,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,254,254,254,255,255,255,255,255,255,255,255,255,255,253,254,255,255,255,254,254,255,255,255,255,255,255,255;255,255,255,255,252,255,223,45,0,4,2,0,0,0,0,0,0,0,0,0,7,6,0,0,0,0,0,0,0,0,0,3,2,0,76,239,255,253,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,255,255,255,253,255,238,96,0,1,3,0,0,0,0,0,0,1,0,28,26,0,1,0,0,0,0,0,0,3,0,11,125,251,254,254,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,255,255,255,255,253,254,255,181,35,0,4,3,1,0,0,0,3,0,77,64,0,4,0,0,0,1,3,4,0,62,217,255,251,254,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,255,255,255,255,255,254,251,255,223,64,0,1,0,1,2,1,0,0,191,158,0,2,1,2,1,0,0,0,92,242,255,252,254,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,255,255,255,255,255,255,255,252,254,250,160,58,12,0,0,1,0,132,255,253,111,2,3,0,0,22,80,186,255,254,253,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,255,255,255,255,255,255,255,255,253,253,255,255,220,180,178,185,207,254,254,254,252,211,189,177,183,235,255,254,253,254,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,255,255,255,255,255,255,255,255,255,254,251,254,255,255,255,255,255,253,255,254,253,255,255,255,255,255,253,252,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;]);
g = uint8([255,255,255,255,255,255,255,255,255,255,255,255,254,252,255,255,255,255,255,255,254,255,255,255,255,255,255,253,252,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,255,255,255,255,255,255,255,255,252,253,255,255,255,224,182,178,176,214,255,243,187,177,177,199,241,255,255,255,253,253,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,255,255,255,255,255,255,253,254,254,255,239,131,54,16,1,1,0,3,199,107,0,2,0,6,24,72,183,255,254,255,253,254,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,255,255,255,255,255,254,255,249,153,71,26,0,1,0,0,0,2,0,90,46,0,2,0,0,0,0,5,38,84,192,255,252,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,255,255,255,255,253,255,210,65,0,0,20,115,21,10,107,98,18,0,66,33,0,59,128,62,0,89,86,0,1,6,113,244,255,254,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,255,255,255,253,255,233,29,0,8,0,78,121,11,5,70,59,18,0,66,32,0,45,67,39,0,59,116,29,0,14,0,102,255,251,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,255,255,255,254,255,100,0,77,10,64,40,0,17,37,73,106,55,0,47,25,9,98,94,53,35,9,0,77,20,52,54,0,175,255,253,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,255,255,252,255,149,0,21,60,0,36,1,88,101,42,49,134,100,0,41,18,38,131,79,29,72,123,35,13,16,18,51,0,20,217,255,253,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,255,255,253,254,50,0,3,0,2,0,66,221,71,0,38,140,20,0,29,22,0,83,107,0,5,185,166,5,0,1,0,5,0,139,255,251,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,255,253,255,224,22,0,48,110,8,21,214,187,15,0,49,211,100,0,32,16,15,197,143,0,0,100,245,116,0,72,101,18,0,88,255,254,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,255,255,253,71,0,0,51,203,25,27,142,35,0,1,2,70,144,13,18,1,78,140,17,0,1,0,106,100,0,132,158,0,8,0,141,255,252,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,252,255,193,0,3,0,34,175,20,10,138,42,0,5,71,147,160,13,13,0,75,176,106,31,0,0,122,88,0,102,141,0,4,0,41,247,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,252,255,149,0,3,9,87,45,0,112,253,134,0,43,209,233,89,0,5,0,21,193,242,125,0,28,224,206,43,0,72,58,0,0,13,213,255,253,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,253,255,95,0,6,133,117,0,83,231,183,37,0,26,188,241,73,0,3,0,8,185,241,88,0,5,91,224,179,10,18,173,70,0,0,160,255,252,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,253,255,209,12,4,0,137,93,0,132,189,49,0,0,17,41,170,158,0,13,0,56,213,89,37,5,0,0,116,190,52,1,154,52,0,0,58,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,252,255,175,0,18,0,72,43,1,48,12,0,33,101,61,0,84,199,3,80,21,99,179,0,29,105,79,9,0,24,32,0,64,21,2,9,21,238,255,254,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,253,255,101,0,5,24,4,1,47,62,0,35,195,243,97,1,134,193,5,75,20,94,204,43,20,192,245,118,0,16,84,28,0,17,21,0,0,170,255,252,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,254,253,253,253,252,254,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,254,253,253,253,252,253,255,255,255,255,255,255,255,255,255,255,255;254,255,247,35,0,59,88,2,125,200,102,0,15,165,244,97,65,245,186,0,30,3,75,255,157,38,172,233,75,0,27,179,186,68,19,111,22,0,110,255,252,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,254,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;254,255,236,25,0,75,57,14,127,203,190,10,0,82,74,0,0,153,186,0,37,5,74,222,47,0,11,105,26,0,86,233,156,73,3,85,33,0,105,255,250,251,251,251,251,251,251,254,255,255,255,255,255,255,255,255,255,253,251,251,251,251,251,252,255,255,252,251,251,251,251,251,251,251,251,251,251,251,251,251,251,255,255,252,251,251,251,251,251,251,251,251,251,251,251,251,251,251,251,252,255,255,255,255,255,255,255,255,254,254,254,255,255,255,255,255,255,255,232,219,217,208,191,224,255,254,255,255,255,255,255,255,255,255,255,255,255,255,255,241,218,219,211,196,208,255,255,255,255,255,255,255,255,255,255,255;254,255,244,33,0,7,5,0,1,171,142,0,3,88,16,2,0,53,175,0,98,37,84,164,0,7,0,61,53,0,31,202,78,0,3,3,4,0,99,255,252,255,255,255,255,255,255,254,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,254,251,254,255,255,255,255,255,253,255,131,20,4,6,0,130,255,252,255,255,255,255,255,255,255,255,255,255,255,253,255,181,40,11,5,0,66,254,254,255,255,255,255,255,255,255,255,255;253,255,201,7,0,6,2,2,13,108,16,0,38,164,32,0,0,86,177,0,85,29,75,183,9,0,0,124,129,0,0,62,76,0,4,6,8,0,48,255,178,83,107,110,110,110,108,224,255,254,255,255,255,255,255,253,255,207,101,111,109,108,93,112,248,255,146,87,108,110,109,107,95,95,95,96,96,100,111,107,121,242,255,144,101,113,105,96,96,96,107,110,107,96,96,96,104,112,102,141,255,254,255,255,255,255,252,255,170,60,237,255,254,255,255,255,255,255,241,214,88,0,0,132,255,252,255,255,255,255,255,255,255,255,255,255,255,255,255,247,229,151,5,0,70,254,254,255,255,255,255,255,255,255,255,255;252,255,182,0,25,42,20,146,126,21,0,0,121,241,205,99,88,199,188,1,56,16,75,233,136,81,153,246,214,44,0,0,65,155,105,7,72,2,22,244,196,86,22,0,1,4,0,139,255,252,255,255,255,255,255,252,255,124,0,4,0,4,72,137,250,255,168,75,13,0,0,18,108,111,107,93,92,45,0,0,23,231,255,58,0,0,18,80,94,100,13,0,10,98,95,86,25,0,0,51,255,254,255,255,255,255,253,255,75,0,169,255,252,255,255,255,255,255,253,255,213,17,0,132,255,252,255,255,255,255,255,255,255,255,255,255,255,255,255,254,255,254,64,0,70,255,254,255,255,255,255,255,255,255,255,255;252,255,100,0,21,9,51,213,137,1,0,55,208,227,208,153,208,255,190,5,72,26,82,251,245,176,170,218,229,155,4,0,42,197,157,5,21,9,0,160,255,255,230,33,15,22,0,53,253,254,255,255,255,255,255,254,254,52,0,10,0,129,255,252,255,255,253,255,198,13,0,50,253,255,254,255,254,255,122,0,32,233,255,68,0,29,216,255,255,246,41,0,35,241,255,255,234,52,0,62,255,254,255,255,255,253,255,207,14,0,76,255,253,255,255,255,255,255,253,255,222,24,0,133,255,252,255,255,255,255,255,255,255,255,255,255,255,255,255,255,253,254,74,0,70,255,254,255,255,255,255,255,255,255,255,255;255,255,53,0,3,0,28,172,166,29,0,57,225,116,14,0,56,217,164,0,96,44,32,227,144,14,0,42,201,159,0,0,98,202,112,0,1,4,0,116,255,245,255,106,51,104,0,4,195,255,252,255,255,255,252,255,190,35,87,0,0,170,255,249,255,255,252,255,233,29,0,45,245,252,251,252,252,253,255,64,14,235,255,68,0,176,255,249,255,241,36,0,31,236,255,249,255,197,0,59,255,254,255,255,255,252,255,124,0,3,12,213,255,253,255,255,255,255,254,255,221,23,0,133,255,252,255,255,255,255,255,254,254,254,254,255,255,255,255,255,254,255,72,0,70,254,254,254,254,254,255,255,255,255,255,255;255,246,34,27,4,3,4,74,185,150,1,0,158,115,0,6,0,65,105,0,79,36,13,112,7,0,0,18,186,62,0,51,209,135,31,0,3,18,19,93,255,249,255,124,50,191,0,0,112,255,252,255,255,255,252,255,91,85,136,0,0,170,255,251,255,255,254,255,229,26,0,45,248,255,254,255,255,252,255,185,35,235,255,45,58,255,252,254,255,244,37,0,31,239,255,254,251,255,87,36,255,254,255,255,254,255,246,43,36,3,0,130,255,252,255,255,255,255,254,255,222,23,0,133,255,252,255,255,255,253,253,255,255,255,255,252,255,255,255,255,254,255,73,0,70,254,251,254,255,255,253,253,255,255,255,255;255,241,33,54,8,128,43,0,119,234,49,0,86,162,0,1,0,30,122,0,54,15,53,105,0,3,0,64,177,15,0,127,222,52,0,97,88,12,54,85,255,249,255,125,36,243,43,0,34,239,255,254,255,254,255,236,17,184,129,0,0,170,255,251,255,255,254,255,231,27,0,45,248,255,254,255,254,237,251,254,234,251,255,136,187,255,253,254,255,244,37,0,31,239,255,254,253,255,203,132,255,254,255,255,252,255,172,33,192,19,0,46,248,255,254,255,255,255,254,255,222,23,0,133,255,252,255,255,253,255,255,243,229,238,253,255,254,255,255,255,254,255,73,0,69,255,255,252,237,245,255,255,253,255,255,255;255,209,21,71,0,132,88,0,157,145,0,0,67,245,144,43,25,126,197,8,52,15,93,209,46,24,74,219,198,9,0,31,199,78,0,131,63,14,71,35,246,254,255,127,31,255,120,0,0,168,255,252,255,252,255,152,29,255,119,0,0,170,255,251,255,255,254,255,231,27,0,45,248,254,253,255,238,45,214,255,254,255,255,255,254,254,255,254,255,244,37,0,31,239,255,254,255,255,255,255,255,255,255,255,253,255,76,90,255,71,0,0,184,255,252,255,255,255,254,255,222,23,0,133,255,252,255,254,255,199,86,34,30,15,41,136,249,255,254,255,254,255,73,0,77,244,125,35,2,20,87,202,255,253,255,255;255,179,7,50,0,36,85,13,148,26,0,0,101,255,255,196,170,241,193,4,43,16,76,244,204,156,218,255,209,26,0,0,103,113,20,91,0,9,41,18,235,255,255,126,33,255,204,5,0,82,255,253,255,253,255,57,115,255,113,0,0,170,255,251,255,255,254,255,231,27,0,46,249,255,252,255,177,1,216,255,253,255,255,252,253,255,255,254,255,244,37,0,31,239,255,254,255,255,253,252,255,255,255,254,255,223,13,188,255,164,0,0,96,255,253,255,255,255,254,255,222,23,0,133,255,252,253,255,157,0,66,217,233,166,13,0,110,255,253,255,254,255,72,0,48,79,77,136,120,35,0,11,170,255,253,255;255,176,0,4,12,0,84,154,129,1,0,12,171,233,142,41,46,205,155,0,19,3,37,226,122,16,53,187,224,98,0,0,46,168,140,36,0,13,0,8,222,255,255,126,35,255,254,62,0,17,219,255,251,255,203,18,210,255,115,0,0,170,255,251,255,255,254,255,231,27,0,44,244,247,250,220,47,0,218,255,253,255,255,255,255,255,255,254,255,244,37,0,31,239,255,254,255,255,255,255,255,255,255,251,255,133,42,252,255,235,35,0,24,225,255,254,255,255,254,255,222,23,0,133,255,250,255,228,21,0,114,255,249,255,137,0,17,216,255,253,254,255,71,0,15,180,255,252,255,237,62,0,13,209,255,253;255,176,0,16,70,41,6,153,201,22,0,60,223,135,1,2,0,67,122,0,9,0,31,137,5,0,0,23,180,170,6,0,88,219,80,0,85,76,0,4,217,255,255,126,35,253,255,146,0,0,141,255,248,255,106,63,255,255,116,0,0,170,255,251,255,255,254,255,231,25,0,7,32,30,25,7,0,7,216,255,253,255,255,255,255,255,255,254,255,244,37,0,31,239,255,254,255,255,255,255,255,255,255,255,255,44,129,255,245,255,111,0,0,150,255,252,255,255,254,255,222,23,0,133,255,250,255,220,12,0,95,255,247,255,190,0,0,179,255,251,254,255,73,0,73,255,252,250,249,255,204,11,0,92,255,253;255,187,0,15,72,136,43,19,187,76,0,13,189,148,0,5,0,56,162,3,10,0,64,156,0,1,0,12,192,112,0,1,158,133,0,102,128,71,0,25,240,255,255,126,35,255,255,224,18,0,56,252,255,245,28,165,255,255,116,0,0,170,255,251,255,255,254,255,231,26,0,34,191,191,180,134,14,5,217,255,253,255,255,255,255,255,255,254,255,244,37,0,31,239,255,254,255,255,255,255,255,255,253,255,187,24,234,255,251,255,213,11,0,61,255,254,255,255,254,255,222,23,0,133,255,252,254,255,167,117,224,255,250,255,194,1,0,172,255,251,254,255,73,0,69,252,254,254,255,251,255,68,0,22,233,255;255,191,0,7,33,14,61,0,135,195,68,0,42,206,66,0,8,177,209,11,7,0,80,246,93,0,0,146,159,0,5,108,212,62,13,58,5,30,0,19,234,255,255,126,35,255,252,255,84,0,6,190,255,161,33,243,254,255,116,0,0,170,255,251,255,255,254,255,231,27,0,48,252,255,251,255,137,0,219,255,254,254,254,255,255,255,255,254,255,244,37,0,31,239,255,254,255,255,255,255,255,255,252,255,96,33,177,165,167,166,167,34,0,6,197,255,253,255,254,255,222,23,0,133,255,252,255,255,255,255,254,255,232,164,56,0,0,173,255,251,254,255,73,0,70,255,254,255,255,251,255,118,0,4,192,255;255,177,0,1,9,11,0,0,86,149,28,0,1,194,93,0,18,212,162,0,5,0,34,230,128,0,0,177,116,0,0,60,145,38,0,1,6,4,0,12,228,255,255,126,36,255,251,255,172,0,0,109,255,65,113,255,248,255,116,0,0,170,255,251,255,255,254,255,231,27,0,45,245,252,250,255,220,12,213,255,253,255,255,255,255,255,255,254,255,244,37,0,31,239,255,254,255,255,255,255,255,254,255,234,26,27,28,26,26,25,29,30,3,0,113,255,252,255,254,255,222,23,0,133,255,252,255,253,255,244,185,86,43,82,124,11,0,173,255,251,254,255,73,0,70,255,254,255,255,252,255,139,0,0,176,255;255,178,0,5,2,0,62,45,0,12,0,0,12,97,5,3,0,123,137,0,3,0,26,178,47,0,0,43,81,0,2,1,8,0,82,38,0,6,1,13,231,255,255,127,34,255,254,255,238,36,0,41,207,22,210,255,250,255,116,0,0,170,255,251,255,255,254,255,231,27,0,45,248,255,254,255,249,162,236,255,255,223,230,255,254,255,255,254,255,244,37,0,31,239,255,254,255,255,255,255,255,252,255,150,28,241,255,252,253,252,255,226,27,0,32,240,255,254,254,255,222,23,0,133,255,252,253,255,188,49,0,88,215,255,200,6,0,173,255,251,254,255,73,0,70,255,254,255,255,252,255,139,0,0,178,255;255,220,10,14,49,0,37,155,55,5,2,0,59,66,7,25,0,103,145,0,3,0,37,177,40,7,21,12,89,25,0,2,16,106,134,0,15,55,0,51,250,253,255,128,26,247,255,250,255,111,0,16,44,77,255,254,252,255,116,0,0,170,255,251,255,255,254,255,231,27,0,46,249,255,254,255,255,255,255,254,255,47,139,255,252,255,255,254,255,244,37,0,31,239,255,254,255,255,255,255,255,255,255,50,106,255,251,255,255,255,252,255,96,0,0,168,255,252,254,255,222,23,0,133,255,250,255,199,13,0,126,255,250,255,189,0,0,173,255,251,254,255,73,0,69,254,254,255,255,251,255,124,0,10,205,255;255,255,47,0,44,75,0,154,206,57,0,4,166,216,139,152,172,215,146,0,3,0,30,203,192,161,137,158,230,95,0,0,109,233,88,9,86,17,0,101,255,249,255,129,21,241,255,252,255,195,0,3,0,166,255,252,252,255,116,0,0,169,255,252,255,255,254,255,230,26,0,44,246,255,254,255,255,252,251,255,184,0,166,255,252,255,255,254,255,243,36,0,31,238,255,254,255,255,255,255,253,255,199,0,191,255,253,255,255,255,252,255,182,0,0,79,255,253,254,255,221,23,0,132,255,248,255,96,0,25,231,255,249,255,193,1,0,172,255,251,254,255,73,0,72,254,254,255,255,251,255,83,0,44,250,255;253,255,79,8,15,104,4,98,237,99,0,0,68,188,201,113,73,72,35,0,1,0,0,54,66,65,137,213,137,22,0,5,153,224,39,37,82,20,3,131,255,245,255,110,0,219,255,249,255,249,57,0,37,241,255,252,248,255,120,0,0,170,255,247,255,255,251,255,233,28,0,38,239,255,251,251,252,252,255,252,62,0,183,255,252,255,255,251,254,244,39,0,27,238,255,250,254,255,255,253,250,255,97,21,250,253,252,255,255,255,254,253,241,43,0,11,212,255,248,255,221,24,0,134,255,249,255,71,0,37,247,254,248,255,191,0,0,176,255,250,254,255,72,0,55,248,253,252,251,255,227,25,0,146,255,252;252,255,160,7,10,77,19,52,194,38,0,10,0,28,109,3,14,22,0,1,0,0,0,0,33,0,50,98,0,17,13,0,86,182,2,55,50,6,14,213,255,255,234,41,0,114,255,254,251,255,140,0,123,255,251,255,254,255,84,0,0,120,255,252,255,255,253,255,214,17,1,11,199,255,253,255,255,253,255,130,0,0,194,255,252,255,255,253,255,225,21,1,5,191,255,253,255,255,255,253,255,186,2,42,244,255,254,255,255,255,255,251,255,104,0,0,92,255,253,255,218,22,0,106,255,249,255,86,0,10,198,255,255,226,64,0,0,154,255,252,254,255,72,0,0,169,255,252,253,255,111,0,73,250,255,254;254,255,246,57,0,30,41,2,109,13,70,188,128,4,11,48,80,158,94,1,1,0,11,129,137,43,40,0,37,175,178,31,48,82,0,54,13,0,109,255,176,80,25,0,2,0,31,103,229,255,215,14,205,255,255,186,101,61,0,1,0,2,63,129,249,255,161,94,35,0,2,0,16,80,88,100,95,80,35,0,1,0,206,255,252,255,241,124,93,27,0,2,0,9,67,119,238,255,255,171,65,5,0,0,33,89,183,255,252,255,242,128,76,11,0,1,0,40,125,143,58,0,0,7,84,204,255,186,6,0,21,99,90,52,178,54,0,19,87,180,255,253,62,6,34,2,123,200,193,88,0,75,239,255,254,255;255,253,255,217,17,0,40,38,2,60,149,226,225,47,0,3,0,31,139,26,0,0,71,146,5,0,3,0,101,243,199,117,37,0,55,22,0,50,244,255,176,107,130,133,132,134,126,118,228,255,250,211,251,254,255,184,103,125,133,132,132,133,117,128,249,255,158,107,128,133,132,133,129,119,118,115,116,120,126,133,132,134,238,255,253,255,241,123,114,128,133,132,133,131,119,121,237,255,255,168,110,134,133,132,128,106,181,255,252,255,242,126,116,130,132,132,134,122,127,121,118,133,132,134,106,195,255,255,182,66,37,37,98,220,255,204,88,66,74,195,255,252,48,123,250,143,64,54,49,60,160,251,254,254,255,255;255,255,252,255,98,0,4,71,35,0,2,61,133,91,91,92,50,52,129,17,0,0,54,134,39,73,98,77,96,114,25,0,0,58,48,0,0,144,255,251,255,255,255,255,255,255,255,255,255,255,255,255,254,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,253,255,251,244,252,255,255,253,255,255,255,255,255,255,255,226,245,255,255,255,244,244,255,255,253,254,255,255,255;255,255,254,255,233,56,0,10,73,92,69,11,0,14,52,149,220,244,120,0,4,0,5,167,239,204,100,20,6,0,34,93,96,56,1,0,108,255,254,255,253,251,251,251,251,251,251,251,254,255,255,253,255,255,255,253,251,251,251,251,251,251,251,251,255,255,252,251,251,251,251,251,251,251,251,251,251,251,251,251,251,251,254,255,255,255,255,251,251,251,251,251,251,251,251,251,254,255,255,253,251,251,251,251,251,251,253,255,255,255,255,251,251,251,251,251,251,251,251,251,251,251,251,251,251,253,255,255,252,253,255,254,251,254,255,253,252,254,251,253,255,255,255,254,254,251,253,255,255,253,252,255,255,255,255,255;255,255,255,254,255,242,60,0,4,51,101,106,51,12,0,4,69,159,90,0,2,0,4,117,129,35,0,3,22,68,115,83,28,2,0,89,255,254,254,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,254,254,254,255,255,255,255,255,255,255,255,255,255,253,254,255,255,255,254,254,255,255,255,255,255,255,255;255,255,255,255,252,255,223,45,0,2,0,11,40,42,15,1,0,4,12,0,7,5,2,9,0,0,3,26,41,27,2,0,1,0,77,239,255,253,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,255,255,255,253,255,238,97,0,3,64,107,18,0,0,16,7,0,0,28,26,0,0,14,12,0,0,36,118,51,0,12,125,252,254,254,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,255,255,255,255,253,254,255,182,31,14,68,16,0,32,108,79,1,0,78,65,0,11,106,83,22,0,30,60,0,61,217,255,251,254,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,255,255,255,255,255,254,251,255,223,60,0,2,0,38,35,8,1,0,191,158,0,4,13,38,24,0,1,0,88,242,255,252,254,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,255,255,255,255,255,255,255,252,254,251,163,58,11,0,0,1,0,132,255,253,111,1,2,0,0,22,81,189,255,254,253,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,255,255,255,255,255,255,255,255,253,253,255,255,221,181,179,185,207,254,254,254,252,211,189,178,184,235,255,254,253,254,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,255,255,255,255,255,255,255,255,255,254,251,254,255,255,255,255,255,253,255,254,253,255,255,255,255,255,253,252,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;]);
b = uint8([255,255,255,255,255,255,255,255,255,255,255,255,254,253,255,255,255,255,255,255,255,255,255,255,255,255,255,254,253,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,255,255,255,255,255,255,255,255,253,254,255,255,255,231,202,196,201,232,255,246,208,198,185,220,246,255,255,255,254,253,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,255,255,255,255,255,255,253,254,255,255,233,174,142,60,45,50,100,174,245,228,128,66,21,111,134,103,202,252,255,255,253,254,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,255,255,255,255,255,254,255,248,178,112,81,107,101,52,103,39,134,220,254,254,168,69,66,116,90,33,91,93,130,213,255,254,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,255,255,255,255,253,255,209,106,70,104,84,6,68,87,34,11,99,222,255,253,146,42,14,64,82,5,35,105,95,126,169,244,255,254,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,255,255,255,254,255,244,125,74,131,176,23,0,109,199,89,35,42,212,255,255,139,1,51,116,202,26,0,74,184,92,88,159,255,253,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,255,255,255,255,253,155,93,31,110,30,73,100,77,141,70,18,48,218,255,255,153,0,35,68,130,102,98,2,83,64,36,92,194,255,253,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,255,255,253,255,173,84,87,41,160,50,131,42,0,31,43,0,0,165,255,246,59,0,12,68,4,8,77,98,88,136,29,128,77,228,255,254,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,255,255,254,255,130,99,113,166,220,154,48,0,3,123,72,0,89,236,254,250,179,13,5,145,57,0,8,88,139,162,112,80,81,199,255,253,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,255,254,255,241,216,66,8,32,109,134,0,1,51,210,120,0,52,242,254,255,157,0,23,212,136,18,0,34,138,33,17,0,105,202,252,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,255,255,245,159,112,66,66,0,105,152,0,51,158,204,155,0,0,168,255,248,57,0,45,224,174,140,0,47,193,11,9,94,53,121,182,255,253,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,253,255,209,150,159,201,158,0,110,111,0,90,248,120,28,1,0,143,255,235,37,0,5,75,203,204,0,29,171,41,25,238,167,178,142,246,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,254,255,215,207,176,164,43,75,131,13,0,5,191,127,0,1,18,204,255,255,100,0,0,15,215,77,0,0,71,119,16,87,158,189,218,234,255,254,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,255,253,191,159,31,0,23,203,59,0,2,11,163,161,0,0,59,255,242,255,171,0,0,47,218,69,1,2,0,108,115,0,0,84,184,206,255,254,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,254,255,220,110,109,174,8,27,141,8,0,50,158,217,95,0,1,21,229,220,251,120,0,0,24,187,196,123,11,0,81,133,0,93,179,78,161,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,253,255,218,107,108,231,41,95,194,17,152,197,108,53,8,58,19,0,185,149,198,70,0,44,28,17,74,159,211,79,80,189,21,154,203,89,176,243,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,254,255,169,137,102,61,68,169,103,0,214,137,0,2,5,38,11,0,151,179,195,46,0,38,32,0,0,13,223,94,2,121,137,58,64,139,151,206,255,253,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,250,136,98,11,15,57,0,3,0,137,195,5,0,10,11,1,0,173,235,232,60,0,4,14,4,0,85,227,26,0,0,22,68,0,75,134,174,255,253,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,243,137,91,1,95,116,0,7,0,26,219,63,12,177,203,21,0,195,227,242,79,0,109,232,108,0,183,136,0,6,0,25,173,29,40,110,179,255,253,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,247,117,166,128,124,224,54,0,0,39,198,31,64,190,247,74,0,193,156,201,81,0,182,222,158,6,118,121,6,0,25,154,244,75,198,122,171,255,254,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;254,255,233,123,152,149,169,137,56,3,125,238,145,0,47,153,152,28,0,201,173,214,84,0,88,158,107,1,22,209,215,65,19,86,149,116,132,152,190,250,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;254,255,228,153,17,61,104,0,0,100,255,225,31,0,1,27,19,0,0,184,207,225,72,0,7,29,10,1,0,113,255,225,25,0,11,138,11,95,219,237,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;254,255,204,155,76,166,74,0,0,145,248,72,0,3,0,0,0,4,0,154,191,203,54,0,3,0,0,0,1,0,168,248,59,0,4,176,142,130,156,201,255,253,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,148,149,250,255,145,0,0,89,232,93,0,0,91,183,80,0,3,212,159,207,108,0,19,147,168,32,0,16,192,192,21,0,22,227,255,233,123,198,255,253,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,254,131,52,213,185,153,14,0,1,85,188,35,0,147,234,255,94,18,248,166,223,160,0,200,245,206,62,0,125,177,22,0,0,76,169,189,141,43,190,255,254,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,172,64,79,0,144,180,0,1,0,164,92,0,94,138,129,82,0,214,195,235,108,0,125,123,136,26,0,215,87,0,0,47,208,43,0,138,64,176,255,254,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,240,132,49,93,0,58,192,4,4,156,250,96,0,7,28,11,1,0,144,201,203,52,0,1,27,21,0,0,204,245,98,0,70,165,0,38,143,46,182,251,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,219,85,96,250,99,5,64,0,120,255,246,50,0,3,0,0,5,0,158,187,199,65,0,3,0,0,4,0,123,255,242,42,9,46,2,204,221,82,148,246,255,254,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,198,124,186,144,214,32,0,0,164,251,132,0,0,20,130,102,1,5,219,196,225,121,0,41,159,96,0,0,32,207,255,83,0,0,134,195,119,213,157,227,255,254,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,193,111,79,0,96,105,0,0,101,205,60,0,1,182,254,251,72,3,232,200,231,135,0,179,250,255,116,0,2,169,192,35,0,33,187,49,0,157,137,220,255,254,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,223,137,63,0,0,116,111,0,10,138,177,0,0,169,141,117,51,0,168,230,227,72,0,103,110,142,98,0,44,198,76,0,17,133,47,0,3,97,166,243,255,254,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,226,134,118,35,143,117,170,2,0,12,192,129,0,1,46,40,0,0,146,253,238,60,0,10,71,21,0,31,199,100,0,0,56,169,93,100,18,143,185,240,255,254,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,226,109,146,146,111,186,179,25,0,107,250,210,0,52,234,145,0,0,194,252,255,105,0,33,215,187,0,73,255,222,64,0,75,184,127,104,161,122,103,235,255,254,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,221,118,78,181,180,36,107,120,138,251,255,186,6,196,227,208,27,9,221,255,255,142,0,113,211,236,108,40,250,255,225,158,139,41,71,220,137,68,109,236,255,254,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,229,114,44,12,134,97,0,26,51,121,255,74,14,108,29,65,18,2,197,255,255,112,0,52,44,55,82,0,151,228,66,64,10,9,141,80,0,91,142,250,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,141,94,54,30,163,17,0,0,96,147,0,1,0,0,0,3,0,161,255,255,84,0,3,0,0,1,0,21,163,29,0,0,58,114,8,94,110,170,255,253,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;254,255,131,82,140,18,107,33,0,0,172,246,66,0,0,80,84,91,138,238,255,255,220,98,101,136,49,0,10,132,255,106,0,0,107,111,48,137,81,199,255,253,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;253,255,188,42,115,15,96,74,0,69,193,143,166,75,0,137,102,80,118,172,255,242,140,100,73,166,79,0,184,175,135,167,22,0,135,70,39,81,59,228,255,254,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;254,255,246,95,89,55,37,95,0,44,33,0,2,135,178,79,65,6,0,100,255,237,37,0,18,64,81,193,76,0,0,33,7,33,105,13,94,100,138,255,254,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,253,255,218,93,126,17,24,53,23,0,2,0,26,117,133,208,95,0,92,255,234,34,0,156,168,102,84,4,0,1,0,50,95,14,66,133,143,247,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,253,255,146,117,139,4,59,132,102,58,3,0,1,0,35,51,0,149,255,255,79,0,61,13,0,1,5,9,85,114,106,37,20,124,102,184,255,253,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,254,255,242,159,130,112,3,8,63,98,77,106,99,10,0,0,13,206,255,255,142,0,3,0,36,102,147,79,77,39,0,22,175,137,149,253,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,255,254,255,240,87,74,105,41,24,1,18,60,113,135,100,21,0,105,255,238,58,0,46,131,132,86,49,9,6,37,66,119,113,154,255,254,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,255,255,253,255,218,92,119,98,132,115,58,52,57,124,252,234,134,119,255,236,108,164,252,226,96,47,38,80,107,112,85,108,143,241,255,254,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,255,255,255,254,255,234,175,86,37,36,28,84,237,128,121,239,254,208,252,243,215,255,212,98,168,210,36,32,34,52,63,155,250,255,254,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,255,255,255,255,253,254,255,200,99,75,103,142,197,57,6,28,103,220,241,248,178,75,19,14,82,217,98,84,67,150,223,255,253,254,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,255,255,255,255,255,254,253,255,228,143,119,134,158,57,107,78,135,191,236,232,163,148,159,82,66,165,112,92,137,239,255,253,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,255,255,255,255,255,255,255,252,255,242,177,106,102,133,112,105,150,216,253,254,200,144,146,93,96,90,130,192,254,254,253,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,255,255,255,255,255,255,255,255,254,254,255,255,228,198,194,214,227,252,255,255,252,227,225,190,199,239,255,255,254,254,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;255,255,255,255,255,255,255,255,255,255,255,254,252,254,255,255,255,255,255,254,255,255,255,255,255,255,255,255,254,253,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255;]);
lablogo = r;
lablogo(:,:,2) = g;
lablogo(:,:,3) = b;

image(lablogo);
axis off
axis image
